title: iOS Privacy: Announcing InAppBrowser.com - see what JavaScript commands get injected through an in-app browser
url: https://krausefx.com/blog/announcing-inappbrowsercom-see-what-javascript-commands-get-executed-in-an-in-app-browser
hash_url: 8e934525d3694413fe73a9af99cdf99d

<p><a href="https://krausefx.com/blog/ios-privacy-instagram-and-facebook-can-track-anything-you-do-on-any-website-in-their-in-app-browser">Last week I published a report</a> on the risks of mobile apps using in-app browsers. Some apps, like Instagram and Facebook, inject JavaScript code into third party websites that cause potential security and privacy risks to the user.</p>

<p>I was so happy to see the article featured by major media outlets across the globe, like <a href="https://www.theguardian.com/technology/2022/aug/11/meta-injecting-code-into-websites-visited-by-its-users-to-track-them-research-says">TheGuardian</a> and <a href="https://www.theregister.com/2022/08/12/meta_ios_privacy/">The Register</a>, generated a <a href="https://twitter.com/KrauseFx/status/1557412468368052225">over a million impressions on Twitter</a>, and was ranked <a href="https://news.ycombinator.com/item?id=32415470">#1 on HackerNews</a> for more than 12 hours. After reading through the replies and DMs, I saw a common question across the community:</p>


<div class="tiktokcontainer" id="desktop">
  <a href="https://krausefx.com/assets/posts/inappbrowser/app_screenshots/tiktok_top.png" target="_blank">
    <img src="https://krausefx.com/assets/posts/inappbrowser/app_screenshots/tiktok_top.png" alt="An iPhone showing the inappbrowser.com website, rendered inside TikTok, showing how there is CSS code being added, added monitoring for all taps and all keyboard inputs, as well as getting the coordinates of elements the user taps">
  </a>
  <h4>TikTok's In-App Browser injecting code to observe all taps and keyboard inputs, which can include passwords and credit cards</h4>
</div>

<p><strong>“How can I verify what apps do in their webviews?”</strong></p>

<p>Introducing <a href="https://InAppBrowser.com">InAppBrowser.com</a>, a simple tool to list the JavaScript commands executed by the iOS app rendering the page.</p>

<p>To try this this tool yourself:</p>

<ol>
  <li>Open an app you want to analyze</li>
  <li>Share the url <a href="https://InAppBrowser.com">https://InAppBrowser.com</a> somewhere inside the app (e.g. send a DM to a friend, or post to your feed)</li>
  <li>Tap on the link inside the app to open it</li>
  <li>Read the report on the screen</li>
</ol>


<div class="tiktokcontainer no-print" id="mobile">
  <a href="https://krausefx.com/assets/posts/inappbrowser/app_screenshots/tiktok_top.png" target="_blank">
    <img src="https://krausefx.com/assets/posts/inappbrowser/app_screenshots/tiktok_top.png" alt="An iPhone showing the inappbrowser.com website, rendered inside TikTok, showing how there is CSS code being added, added monitoring for all taps and all keyboard inputs, as well as getting the coordinates of elements the user taps">
  </a>
  <h4>TikTok's In-App Browser injecting code to observe all taps and keyboard inputs, which can include passwords and credit cards</h4>
</div>

<p>I started using this tool to analyze the most popular iOS apps that have their own in-app browser. Below are the results I’ve found.</p>

<p>For this analysis I have excluded all third party iOS browsers (Chrome, Brave, etc.), as they use JavaScript to offer some of their functionality, like a password manager. Apple requires all third party iOS browsers apps to use the Safari rendering engine <code class="language-plaintext highlighter-rouge">WebKit</code>.</p>

<p><strong><em>Important Note:</em></strong> This tool can’t detect all JavaScript commands executed, as well as doesn’t show any tracking the app might do using native code (like custom gesture recognisers). More details on this below.</p>

<p><strong>Fully Open Source</strong></p>

<p><a href="https://InAppBrowser.com">InAppBrowser.com</a> is designed for everybody to verify for themselves what apps are doing inside their in-app browsers. I have decided to open source the code used for this analysis, you can check it out on <a href="https://github.com/KrauseFx/inAppBrowser.com">GitHub</a>. This allows the community to update and improve this script over time.</p>

<h2 id="ios-apps-that-have-their-own-in-app-browser">iOS Apps that have their own In-App Browser</h2>

<ul>
  <li><strong>Option to open in default browser</strong>: Does the app provide a button to open the currently shown link in the default browser?</li>
  <li><strong>Modify page</strong>: Does the app inject JavaScript code into third party websites to modify its content? This includes adding tracking code (like inputs, text selections, taps, etc.), injecting external JavaScript files, as well as creating new HTML elements.</li>
  <li><strong>Fetch metadata</strong>: Does the app run JavaScript code to fetch website metadata? This is a harmless thing to do, and doesn’t cause any real security or privacy risks.</li>
  <li><strong>JS</strong>: A link to the JavaScript code that I was able to detect. Disclaimer: There might be other code executed. The code might not be a 100% accurate representation of all JS commands.</li>
</ul>



<p>Click on the <code class="language-plaintext highlighter-rouge">Yes</code> or <code class="language-plaintext highlighter-rouge">None</code> on the above table to see a screenshot of the app.</p>

<p><strong>Important</strong>: Just because an app injects JavaScript into external websites, doesn’t mean the app is doing anything malicious. There is no way for us to know the full details on what kind of data each in-app browser collects, or how or if the data is being transferred or used. This publication is stating the JavaScript commands that get executed by each app, as well as describing what effect each of those commands might have. For more background on the risks of in-app browsers, check out <a href="https://krausefx.com/blog/ios-privacy-instagram-and-facebook-can-track-anything-you-do-on-any-website-in-their-in-app-browser">last week’s publication</a>.</p>

<p>Even if some of the apps above have green checkmarks, they might use the new <code class="language-plaintext highlighter-rouge">WKContentWorld</code> isolated JavaScript, which I’ll describe below.</p>

<h2 id="tiktok-monitoring-all-keyboard-inputs-and-taps">TikTok monitoring all keyboard inputs and taps</h2>

<p><a href="https://krausefx.com/assets/posts/inappbrowser/app_screenshots/tiktok.png" target="_blank">
  <img class="details-for-specific-app" src="https://krausefx.com/assets/posts/inappbrowser/app_screenshots/tiktok_cut.png">
</a></p>

<p>When you open any link on the TikTok iOS app, it’s opened inside their in-app browser. While you are interacting with the website, <strong>TikTok subscribes to all keyboard inputs</strong> (including passwords, credit card information, etc.) and every tap on the screen, like which buttons and links you click.</p>

<ul>
  <li>TikTok iOS subscribes to every keystroke (text inputs) happening on third party websites rendered inside the TikTok app. This can include passwords, credit card information and other sensitive user data. (<code class="language-plaintext highlighter-rouge">keypress</code> and <code class="language-plaintext highlighter-rouge">keydown</code>). We can’t know what TikTok uses the subscription for, but from a technical perspective, <strong>this is the equivalent of installing a keylogger</strong> on third party websites.</li>
  <li>TikTok iOS subscribes to every tap on any button, link, image or other component on websites rendered inside the TikTok app.</li>
  <li>TikTok iOS uses a JavaScript function to get details about the element the user clicked on, like an image (<code class="language-plaintext highlighter-rouge">document.elementFromPoint</code>)</li>
</ul>

<p><a href="https://krausefx.com/assets/posts/inappbrowser/app_js/tiktok.js" target="_blank">Here</a> is a list of all JavaScript commands I was able to detect.</p>

<p><strong>Update:</strong> TikTok’s statement, as reported per <a href="https://www.forbes.com/sites/richardnieva/2022/08/18/tiktok-in-app-browser-research/">Forbes.com</a>:</p>

<blockquote>
  <p><strong>The company confirmed those features exist</strong> in the code, but said TikTok is not using them.</p>

  <p>“Like other platforms, we use an in-app browser to provide an optimal user experience, but the Javascript code in question is used only for debugging, troubleshooting and performance monitoring of that experience — like checking how quickly a page loads or whether it crashes,” spokesperson Maureen Shanahan said in a statement.</p>
</blockquote>

<p>The above statement confirms my findings. TikTok injects code into third party websites through their in-app browsers that behaves like a keylogger. However claims it’s not being used.</p>

<h2 id="instagram-does-more-than-just-inserting-pcmjs">Instagram does more than just inserting <code class="language-plaintext highlighter-rouge">pcm.js</code></h2>

<p><a href="https://krausefx.com/assets/posts/inappbrowser/app_screenshots/instagram.png" target="_blank">
  <img class="details-for-specific-app" src="https://krausefx.com/assets/posts/inappbrowser/app_screenshots/instagram_cut.png">
</a></p>

<p><a href="https://krausefx.com/blog/ios-privacy-instagram-and-facebook-can-track-anything-you-do-on-any-website-in-their-in-app-browser">Last week’s post</a> talked about how Meta injects the <a href="https://connect.facebook.net/en_US/pcm.js"><code class="language-plaintext highlighter-rouge">pcm.js</code></a> script onto third party websites. <a href="https://twitter.com/KrauseFx/status/1558867249691123712">Meta claimed</a> they only inject the script to respect the user’s ATT choice, and additional “security and user features”.</p>

<blockquote>
  <p>The code in question allows us to respect people’s privacy choices by helping aggregate events (such as making a purchase online) from pixels already on websites, before those events are used for advertising or measurement purposes.</p>
</blockquote>

<p>– <a href="https://twitter.com/andymstone/status/1557825414176940035">via this tweet</a></p>

<p>After improving the JavaScript detection, <strong>I now found some additional commands Instagram executes:</strong></p>

<ul>
  <li>Instagram iOS subscribes to every tap on any button, link, image or other component on external websites rendered inside the Instagram app.</li>
  <li>Instagram iOS subscribes to every time the user selects a UI element (like a text field) on third party websites rendered inside the Instagram app.</li>
</ul>

<p><a href="https://krausefx.com/assets/posts/inappbrowser/app_js/instagram.js" target="_blank">Here</a> is a list of all JavaScript commands I was able to detect.</p>

<p><strong>Note on subscribing</strong>: When I talk about “<em>App subscribes to</em>”, I mean that the app subscribes to the JavaScript events of that type (e.g. all taps). There is no way to verify what happens with the data.</p>



<p>Since iOS 14.3 (December 2020), Apple introduced the support of running JavaScript code in the <a href="https://developer.apple.com/documentation/webkit/wkcontentworld">context of a specified frame and content world</a>. JavaScript commands executed using this approach can still fully access the third party website, but can’t be detected by the website itself (in this case a tool like InAppBrowser.com).</p>

<blockquote>
  <p>Use a WKContentWorld object as a namespace to separate your app’s web environment from the environment of individual webpages or scripts you execute. Content worlds help prevent issues that occur when two scripts modify environment variables in conflicting ways. […]
Changes you make to the DOM are visible to all script code, regardless of content world.</p>
</blockquote>

<p>– <a href="https://developer.apple.com/documentation/webkit/wkcontentworld">Apple WKContentWorld Docs</a></p>

<p>This new system was initially built so that website operators can’t interfere with JavaScript code of browser plugins, and to make fingerprinting more difficult. As a user, you can check the source code of any browser plugin, as you are in control over the browser itself. However with in-app browsers we don’t have a reliable way to verify all the code that is executed.</p>

<p>So when Meta or TikTok want to hide the JavaScript commands they execute on third party websites, all they’d need to do is to update their JavaScript runner:</p>

<div class="language-swift highlighter-rouge"><div class="highlight"><pre class="highlight"><code><span class="c1">// Currently used code by Meta &amp; TikTok</span>
<span class="k">self</span><span class="o">.</span><span class="nf">evaluateJavaScript</span><span class="p">(</span><span class="n">javascript</span><span class="p">)</span>

<span class="c1">// Updated to use the new system</span>
<span class="k">self</span><span class="o">.</span><span class="nf">evaluateJavaScript</span><span class="p">(</span><span class="n">javascript</span><span class="p">,</span> <span class="nv">in</span><span class="p">:</span> <span class="kc">nil</span><span class="p">,</span> <span class="nv">in</span><span class="p">:</span> <span class="o">.</span><span class="n">defaultClient</span><span class="p">,</span> <span class="nv">completionHandler</span><span class="p">:</span> <span class="p">{</span> <span class="n">_</span> <span class="k">in</span> <span class="p">})</span>
</code></pre></div></div>

<p>For example, Firefox for iOS already <a href="https://github.com/mozilla-mobile/firefox-ios/blob/d613cb24d5dc717466f098e13625a3e0c5e4703e/Shared/Extensions/WKWebViewExtensions.swift#L19-L29">uses the new WKContentWorld system</a>. Due to the open source nature of Firefox and Google Chrome for iOS it’s easy for us as a community to verify nothing suspicious is happening.</p>

<p>Especially after the publicity of <a href="https://krausefx.com/blog/ios-privacy-instagram-and-facebook-can-track-anything-you-do-on-any-website-in-their-in-app-browser">last week’s post</a>, as well as this one, tech companies that still use custom in-app browsers will very quickly update to use the new <code class="language-plaintext highlighter-rouge">WKContentWorld</code> isolated JavaScript system, so their code becomes undetectable to us.</p>

<p>Hence, <strong>it becomes more important than ever to find a solution to end the use of custom in-app browsers</strong> for showing third party content.</p>

<h3 id="valid-use-cases-for-in-app-webviews">Valid use-cases for in-app webviews</h3>

<p>There are many valid reasons to use an in-app browser, particularly when an app accesses its own websites to complete specific transactions. For example, an airline app might not have the seat selection implemented natively for their whole airplane fleet. Instead they might choose to reuse the web-interface they already have. If they weren’t able to inject cookies or JavaScript commands inside their webview, the user would have to re-login while using the app, just so they can select their seat. Shoutout to Venmo, which uses their own in-app browser for all their internal websites (e.g. Terms of Service), but as soon as you tap on an external link, they automatically transition over to <code class="language-plaintext highlighter-rouge">SFSafariViewController</code>.</p>

<p>However, there are data privacy &amp; integrity issues when you use in-app browsers to visit non-first party websites, such as how Instagram and TikTok show all external websites inside their app. More importantly, those apps rarely offer an option to use a standard browser as default, instead of the in-app browser. And in some cases (like TikTok), there is no button to open the currently shown page in the default browser.</p>

<h2 id="ios-apps-that-use-safari">iOS Apps that use Safari</h2>

<p>The apps below follow Apple’s recommendation of using Safari or <code class="language-plaintext highlighter-rouge">SFSafariViewController</code> for viewing external websites. More context on <code class="language-plaintext highlighter-rouge">SFSafariViewController</code> in the <a href="https://krausefx.com/blog/ios-privacy-instagram-and-facebook-can-track-anything-you-do-on-any-website-in-their-in-app-browser">original article</a>.</p>

<p>All apps that use <code class="language-plaintext highlighter-rouge">SFSafariViewController</code> or <code class="language-plaintext highlighter-rouge">Default Browser</code> are on the safe side, and there is no way for apps to inject any code onto websites, even with the new <code class="language-plaintext highlighter-rouge">WKContentWorld</code> system.</p>



<h2 id="what-can-we-do">What can we do?</h2>

<p><strong>As a user of an app</strong></p>

<div id="youtube-button">
  <img src="https://krausefx.com/assets/posts/inappbrowser/app_screenshots/video-preview.png" alt="A link to the YouTube video showing the website in action inside the Instagram app">
  <p><b>Demo video of how to escape the Instagram In-App Browser</b></p>
</div>

<p>Most in-app browsers have a way to open the currently shown website in Safari. As soon as you land inside an in-app browser, use the <code class="language-plaintext highlighter-rouge">Open in Browser</code> feature to switch to a safer browser. If that button isn’t available, you will have to copy &amp; paste the URL to open the link in the browser of your choice. If the app makes it difficult to even do that, you can tap &amp; hold a link on the website and then use the Copy feature, which can be a little tricky to get right.</p>

<p>TikTok doesn’t have a button to open websites in the default browser.</p>

<p><strong>Update:</strong> According to some tweets, sometimes there is a way to open websites in the default browser.</p>

<p><strong>Companies using in-app browsers</strong></p>

<p>If you’re at a company where you have an in-app browser, use it only for your own pages and open all external links in the user’s default browser. Additionally, provide a setting to let users choose a default browser over an in-app browser experience. Unfortunately, these types of changes rarely get prioritized over features that move metrics inside of tech organizations. However, it’s so important for people to educate others on their team, and their managers, about the positive impact of making better security and privacy decisions for the user. These changes can be transparently marketed to users as an opportunity to build further trust.</p>

<p><strong>Major tech companies</strong></p>

<p>It’s important to call out how much movement there’s been in the privacy of data space, but it’s unclear how many of these changes have been motion vs. true progress for the industry and the user.</p>

<blockquote>
  <p>“Many tech companies take heat for ‘abusing their users’ privacy’, when in fact they try to balance out business priorities, great user experiences, and ensuring they are respecting privacy and user data. It’s clear why companies were motivated to provide an in-app experience for external websites in the first place.</p>

  <p>With the latest technology, companies can start to provide a smooth experience for the user, while respecting their privacy. It’s possible for iOS or Android developers to move the privacy standards and responsibility to Apple &amp; Google (e.g. stricter app reviews, more permission screens, etc.), however this is a much larger conversation where companies need to work together to define what standards should exist. We can’t have one or two companies set the direction for the entire industry, since a solution needs to work for the large majority of companies. Otherwise, we’re left in a world where companies are forced to get creative on finding ways to track additional user data from any source possible, or define their own standards of what’s best for user privacy, ultimately hurting the consumer and the product experience.”</p>
</blockquote>

<p>– <a href="https://twitter.com/hemal">Hemal Shah</a></p>

<p>Technology-wise <a href="https://webkit.org/blog/10882/app-bound-domains/">App-Bound Domains</a> seems to be an excellent new WebKit feature making it possible for developers to offer a safer in-app browsing experience when using <code class="language-plaintext highlighter-rouge">WKWebView</code>. As an app developer, you can define which domains your app can access (your own), and you won’t be able to control third party pages any more. To disable the protection, a user would have to explicitly disable it in the iOS settings app. However, at the time of writing, this system is not yet enabled by default.</p>

<h3 id="faqs-for-non-tech-readers">FAQs for non-tech readers</h3>

<ul>
  <li><strong>Can in-app browsers read everything I do online?</strong> Yes, if you are browsing through their in-app browser they technically can.</li>
  <li><strong>Do the apps above actually steal my passwords, address and credit card numbers?</strong> No! I wanted to showcase that bad actors could get access to this data with this approach. As shown in the past, if it’s possible for a company to get access to data legally and for free, without asking the user for permission, <a href="https://twitter.com/steipete/status/1025024813889478656">they will track it</a>.</li>
  <li><strong>How can I protect myself?</strong> Whenever you open a link from any app, see if the app offers a way to open the currently shown website in your default browser. During this analysis, every app besides TikTok offered a way to do this.</li>
  <li><strong>Are companies doing this on purpose?</strong> Building your own in-app browser takes a non-trivial time to program and maintain, significantly more than just using the privacy and user-friendly alternative that’s already been built into the iPhone for the past 7 years. Most likely there is some motivation there for the company to track your activities on those websites.</li>
  <li><strong>I opened InAppBrowser.com inside an app, and it doesn’t show any commands. Am I safe?</strong> No! First of all, the website only checks for one of many hundreds of attack vectors: JavaScript injection from the app itself. And even for those, as of December 2020, app developers can completely hide the JavaScript commands they execute, therefore there is no way for us to verify what is actually happening under the hood.</li>
</ul>
