title: The Market for Lemons
url: https://infrequently.org/2023/02/the-market-for-lemons/
hash_url: 5b35e3f3639ceb7d9f684aa81979f304

<p>For most of the past decade, I have spent a considerable fraction of my professional life consulting with teams building on the web.</p>
<p>It is not going well.</p>
<p>Not only are new services being built to a self-defeatingly low UX and performance standard, existing experiences are pervasively re-developed on unspeakably slow, JS-taxed stacks. At a business level, this is a disaster, raising the question: <em>"why are new teams buying into stacks that have failed so often before?"</em></p>
<p>In other words, <em>"why is this market so inefficient?"</em></p>
<p></p>
<p>George Akerlof's most famous paper introduced economists to the idea that <em>information asymmetries</em> distort markets and reduce the quality of goods because sellers with more information can pass off low-quality items as more valuable than informed buyers appraise them to be. (<a href="https://www.sfu.ca/~wainwrig/Econ400/akerlof.pdf">PDF</a>, <a href="https://en.wikipedia.org/wiki/The_Market_for_Lemons">summary</a>)</p>
<p>Customers that can't assess the quality of products pay the wrong amount for them, creating a disincentive for high-quality products to emerge and working against their success when they do. For many years, this effect has dominated the frontend technology market. Partisans for slow, complex frameworks have successfully marketed lemons as the hot new thing, despite the pervasive failures in their wake, and crowding out higher-quality options in the process.</p>
<p>These technologies were initially pitched on the back of <em>"better user experiences"</em>, but have <a href="https://dev.to/tigt/making-the-worlds-fastest-website-and-other-mistakes-56na">utterly failed</a> to deliver on that promise outside of the <a href="https://infrequently.org/2022/05/performance-management-maturity/">high-management-maturity organisations</a> in which they were born. Transplanted into the wider web, these new stacks have proven to be <a href="https://infrequently.org/2022/12/performance-baseline-2023/">expensive duds</a>.</p>
<p>The complexity merchants knew their environments weren't typical, but they sold highly specialised tools as though they were generally appropriate. They understood that most websites lack tight latency budgeting, dedicated performance teams, hawkish management reviews, ship gates to prevent regressions, and end-to-end measurements of critical user journeys. They understood the only way to scale JS-driven frontends are massive investments in controlling complexity, but warned none of their customers.</p>
<p>They also knew that their choices were hard to replicate. Few can afford to build and maintain 3+ versions of the same web app ("desktop", "mobile", and "lite"), and vanishingly few scaled sites feature long sessions and login-gated content.</p>
<p>Armed with all of this background and knowledge, they kept all the caveats to themselves.</p>
<h2 id="what-did-they-know-and-when-did-they-know-it%3F">What Did They Know And When Did They Know It? <a class="permalink" href="#what-did-they-know-and-when-did-they-know-it%3F">#</a></h2>
<p>This information asymmetry persists; the worst actors still haven't levelled with their communities about what it takes to operate complex JS stacks at scale. They did not signpost the delicate balance of engineering constraints that allowed their products to adopt this new, slow, and complicated tech. Why? For the same reason used car dealers don't talk up average monthly repair costs.</p>
<p>The market for lemons depends on customers having less information than those selling shoddy products. Some who hyped these stacks early on were earnestly ignorant, which is forgivable when recognition of error leads to changes in behaviour. But that's not what the most popular frameworks of the last decade did. As time passed, and the results continued to underwhelm, an initial lack of clarity was parlayed into intentional acts of omission. These omissions have been both material to users and developers. Given the extensive evidence of these failures provided direct to their marketeers, often by me, at some point the omissions veered into intentional prevarication.</p>
<p>Faced with the dawning realisation that this tech mostly made things worse, not better, the JS-industrial-complex <a href="https://www.scientificamerican.com/article/exxon-knew-about-climate-change-almost-40-years-ago/">pulled an Exxon</a>. They <em>could</em> have copped to an honest error, admitted that these technologies require vast infrastructure and investment to operate, and that they are unscalable in the hands of all but the most sophisticated teams. They did the opposite, doubling down, <a href="https://techcrunch.com/2017/04/18/facebook-announces-react-fiber-a-rewrite-of-its-react-framework/">breathlessly announcing vapourware</a> year after <a href="https://www.youtube.com/watch?v=NZoRlVi3MjQ">year</a> to defer critical thinking about the fundamental errors in the design. They also worked behind the scenes to marginalise those who pointed out the disturbing results and extraordinary costs.</p>
<p>Credit where it's due, the complexity merchants have been incredibly effective in one regard: top-shelf marketing discipline. Over the last ten years, they have worked overtime to make frontend an evidence-free zone. The hucksters knew that discussions about performance tradeoffs would not end with teams investing more in their technology, so boosterism and misdirection were aggressively substituted for evidence and debate. Like a curtain of Halon descending to put out the fire of engineering debate, they blanketed the discourse with toxic positivity. Those who dared speak up were branded "negative" and "haters", no matter how much data they lugged in tow.</p>
<h2 id="sandy-foundations">Sandy Foundations <a class="permalink" href="#sandy-foundations">#</a></h2>
<p><a href="https://ericwbailey.website/published/modern-health-frameworks-performance-and-harm/">It was, of course, bullshit.</a></p>
<p>Astonishingly, gobsmackingly effective bullshit, but nonsense nonetheless. There was a point to it, though. Playing for time allowed the bullshitters to punt introspection of the always-wrong assumptions they'd built their entire technical ediface on. In time, these misapprehensions would become cursed articles of faith:</p>
<ul>
<li>CPUs get faster every year <br><br>[ <em>narrator: <a href="https://infrequently.org/2022/12/performance-baseline-2023/">they do not</a></em>  ]<br><br></li>
<li>Organisations can manage these complex stacks <br><br>[ <em>narrator: <a href="https://infrequently.org/2022/05/performance-management-maturity/">they cannot</a></em>  ]</li>
</ul>
<p>All of this was <a href="https://www.youtube.com/watch?v=4bZvq3nodf4">falsified by 2016</a>, but nobody wanted to turn on the house lights while the JS party was in full swing. Not the developers being showered with shiny tools and boffo praise for replacing "legacy" HTML and CSS that performed fine. Not the scoundrels peddling foul JavaScript elixirs and potions. Not the managers that craved a check to write and a rewrite to take credit for in lieu of critical thinking about user needs and market research.</p>
<p>Consider the narrative <a href="https://en.wikipedia.org/wiki/Baffles_(submarine)">Crazy Ivans</a> that led to this point.</p>
<p><lite-youtube videoid="Ar9R-CX217o" videostartat="232" videotitle="2013: Single page, JavaScript-driven applications are the future."></lite-youtube></p>
<p>It's challenging to summarise a vast discourse over the span of a decade, particularly one as dense with jargon and acronyms as the one that led to today's <em>status quo</em> of overpriced failure. These are not quotes, but vignettes of what I have perceived as distinct epochs of our tortured journey:</p>
<ul>
<li><em>"SPAs are a better user experience, and managing state is a big problem on the client side. You'll need a tool to help structure that complexity when rendering on the client side, and our framework works at scale"</em> <br><br>[  <a href="https://reactjs.org/blog/2014/02/15/community-roundup-16.html">illustrative example</a>  ]<br><br></li>
<li><em>"Instead of waiting on the JavaScript that will absolutely deliver a superior SPA experience...someday...why not render on the server as well, so that there's something for the user to look at while they wait for our awesome and <em>totally</em> scalable JavaScript to collect its thoughts?"</em> <br><br>[  <a href="https://webbylab.com/blog/isomorphic-react/">an intro to "isomorphic javascript", a.k.a. "Server-Side Rendering", a.k.a. "SSR"</a>  ]<br><br></li>
<li><em>"SPAs are a better experience, but everyone knows you'll need to do all the work twice because SSR makes that better experience minimally usable. But even with SSR, you might be sending so much JS that things feel bad. So give us credit for a promise of vapourware for delay-loading parts of your JS."</em> <br><br>[  <a href="https://reactjs.org/blog/2018/11/13/react-conf-recap.html">impressive stage management</a>  ]<br><br></li>
<li><em>"SPAs are a better experience. SSR is vital because SPAs take a long time to start up, and you aren't using our vapourware to split your code effectively. As a result, the main thread is often locked up, which could be bad?<br><br>Anyway, this is totally your fault and not the predictable result of us failing to advise you about the controls and budgets we found necessary to scale JS in our environment. Regardless, we see that you lock up main threads for seconds when using our slow system, so in a few years we'll create a parallel scheduler that will break up the work transparently*"</em> <br><br>[  <a href="https://www.youtube.com/watch?v=ZCuYPiUIONs">2017's beautiful overview of a cursed errand</a> and <a href="https://www.youtube.com/watch?v=ByBPyMBTzM0">2018's breathless re-brand</a>  ]<br><br></li>
<li><em>"The scheduler isn't ready, but thanks for your patience; here's a new way to spell your component that introduces new timing issues but doesn't address the fact that our system is incredibly slow, built for browsers you no longer support, and that CPUs are not getting faster"</em> <br><br>[  <a href="https://www.youtube.com/watch?v=wXLf18DsV-I">representative pitch</a>  ]<br><br></li>
<li><em>"Now that you're 'SSR'ing your SPA and have re-spelt all of your components, and given that the scheduler hasn't fixed things and CPUs haven't gotten faster, why not skip SPAs and settle for progressive enhancement of sections of a document?"</em> <br><br>[  <a href="https://www.patterns.dev/posts/islands-architecture/">"islands"</a>, <a href="https://reactjs.org/blog/2020/12/21/data-fetching-with-react-server-components.html">"server components"</a>, etc.  ]</li>
</ul>
<p>The <a href="https://simpsons.fandom.com/wiki/Steamed_Hams">Steamed Hams</a> of technology pitches.</p>
<p>Like Chalmers, many teams and managers acquiesce to the contriditions embedded in the stacked rationalisations. There are dozens of reasons to look the other way, real and imaginary. But even as the genuinely well-intentioned victims of the complexity merchants meekly recapitulate how trickle-down UX will work this time — <em>if only we try it hard enough!</em> — the evidence mounts that "modern" web development is, in the main, an expensive failure.</p>
<p>The baroque and insular terminology of the in-group is a clue. It's functional purpose (outside of signaling) is to obscure furious plate spinning. This tech isn't working for most adopters, but admitting as much would shrink the market for lemons. You'd be forgiven for thinking the verbiage was designed obfuscate. Little comfort, then, that folks selling new approaches must now <a href="https://dev.to/tigt/making-the-worlds-fastest-website-and-other-mistakes-56na">wade through</a> waist-deep jargon excrement <a href="https://www.epicweb.dev/the-webs-next-transition">to argue for the <em>next</em> increment of complexity.</a></p>
<p>The most recent turn is as predictable as it is bilious. Today's most successful complexity merchants have never backed down, never apologised, and never come clean about what they knew about the level of expense involved in keeping SPA-oriented technologies in check. But they expect you'll follow them down the <em>next</em> dark alley anyway:</p>
<figure>
<a href="https://twitter.com/rauchg/status/1619492334961569792" alt="An admission against interest.">
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2023/02/the-market-for-lemons/lemony_snicker.png 2400w,
                  https://infrequently.org/2023/02/the-market-for-lemons/lemony_snicker.png 1600w,
                  https://infrequently.org/2023/02/the-market-for-lemons/lemony_snicker.png 1200w,
                  https://infrequently.org/2023/02/the-market-for-lemons/lemony_snicker.png?nf_resize=fit&amp;w=800   800w,
                  https://infrequently.org/2023/02/the-market-for-lemons/lemony_snicker.png?nf_resize=fit&amp;w=600   600w,
                  https://infrequently.org/2023/02/the-market-for-lemons/lemony_snicker.png?nf_resize=fit&amp;w=500   500w,
                  https://infrequently.org/2023/02/the-market-for-lemons/lemony_snicker.png?nf_resize=fit&amp;w=400   400w">
<img src="https://infrequently.org/2023/02/the-market-for-lemons/lemony_snicker.png" alt="An admission against interest." class="preview" decoding="async" loading="lazy">
</source></picture>

</a>
  <figcaption>An admission against interest.</figcaption>
</figure>
<p>And why not? The substitution of heroic developer narratives for user success happened imperceptibly. Admitting it was a mistake would embarrass the good and the great alike. Once the lemon sellers embed the data-light idea that improved "Developer Experience" ("DX") leads to better user outcomes, improving "DX" became and end unto itself, and many who knew better felt forced to play along. The long lead times in falsifying trickle-down UX was a feature, not a bug; they don't need you to succeed, only to keep buying.</p>
<p>As marketing goes, the "DX" <a href="https://infrequently.org/2018/09/the-developer-experience-bait-and-switch/">bait-and-switch</a> is brilliant, but the tech isn't delivering for anyone <em>but</em> developers. The goal of the complexity merchants is to put your brand on their marketing page and showcase microsite and to make acqui-hiring your failing startup easier.</p>
<h2 id="denouement">Denouement <a class="permalink" href="#denouement">#</a></h2>
<p>After more than a decade of JS hot air, the framework-centric pitch is still phrased in speculative terms because there's no there there. The complexity merchants can't cop to the fact that <a href="https://infrequently.org/2022/05/performance-management-maturity/">management competence and <em>lower</em> complexity</a> — not baroque technology — are determinative of end-user success.</p>
<p>By turns, the simmering embarrassment of a widespread failure of technology-first approaches has created new pressures that have forced the JS colporteurs into a simulated annealing process. In each iteration, they must accept a smaller and smaller rhetorical lane as their sales grow, but the user outcomes fail to improve.</p>
<p>The excuses are running out.</p>
<p>At long last, the journey has culminated with the rollout of <a href="https://web.dev/vitals/">Core Web Vitals</a>. It finally provides an effortless, objective quality measurement prospective customers can use to assess frontend architectures. It's no coincidence the final turn away from the SPA justification has happened just as buyers can see a linkage between the stacks they've bought and the monetary outcomes they already value, namely SEO. The objective buyer, circa 2023, will understand heavy JS stacks as a regrettable legacy, one that teams who have hollowed out their HTML and CSS skill bases will pay dearly for in years to come.</p>
<p>No doubt, many folks who now know their web stacks are slow and outdated will do as Akerlof predicts, and work to obfuscate that reality for managers and customers for as long as possible. The market for lemons is, indeed, mostly a <em>resale</em> market, and the excesses of our lost decade will not be flushed from the ecosystem quickly. Beware tools pitching <em>"100 on Lighthouse"</em> without checking the real-world <a href="https://treo.sh/sitespeed">Core Web Vitals results</a>.</p>
<h2 id="shrinkage">Shrinkage <a class="permalink" href="#shrinkage">#</a></h2>
<p>A subtle aspect of Akerlof's theory is that markets in which lemons dominate eventually shrink. I've <a href="https://infrequently.org/series/performance-inequality/">warned for years</a> that the mobile web is under threat from within, and <a href="https://vimeo.com/364402896">the depressing data I've cited about users moving to apps</a> and away from terrible web experiences is in complete alignment with the theory.</p>
<p>More prosaically, when websites feel like worse experiences to those who greenlight digital services, why should anyone expect them to spend a lot to build a website? And when websites stop being where most of the information and services are, who will hire web developers?</p>
<p>The lost decade we've suffered at the hands of lemon purveyors isn't just a local product travesty; it's also an ecosystem-level risk. Forget AI putting web developers out of jobs; JS-heavy web stacks have been shrinking the future market for your services <em>for years</em>.</p>
<p>As <a href="https://www.theguardian.com/education/2002/dec/20/highereducation.uk1#:~:text=Adam%20Smith's%20invisible%20hand%20%2D%20the,because%20it%20is%20not%20there.">Stigliz memorably quipped</a>:</p>
<blockquote>
<p>Adam Smith's invisible hand — the idea that free markets lead to efficiency as if guided by unseen forces — is invisible, at least in part, because it is not there.</p>
</blockquote>
<p>But dreams die hard.</p>
<p>I'm already hearing laments from folks who have been responsible citizens of framework-landia lo these many years. Oppressed as they were by the lemon vendors, they worry about babies being throw out with the bathwater, and I empathise. But for the sake of users, and for the new opportunities for the web that will open up when experiences finally improve, I say <em>"chuck those tubs"</em>. Chuck 'em hard, and post the photos of the unrepentant bastards that sold this nonsense behind the cash register.</p>
<picture class="preview">
  <source sizes="(max-width: 1200px) 70vw, 600px" srcset="https://infrequently.org/2023/02/the-market-for-lemons/anti-js-js-club-lighter-small.jpg 2400w,
                  https://infrequently.org/2023/02/the-market-for-lemons/anti-js-js-club-lighter-small.jpg 1600w,
                  https://infrequently.org/2023/02/the-market-for-lemons/anti-js-js-club-lighter-small.jpg 1200w,
                  https://infrequently.org/2023/02/the-market-for-lemons/anti-js-js-club-lighter-small.jpg?nf_resize=fit&amp;w=800   800w,
                  https://infrequently.org/2023/02/the-market-for-lemons/anti-js-js-club-lighter-small.jpg?nf_resize=fit&amp;w=600   600w,
                  https://infrequently.org/2023/02/the-market-for-lemons/anti-js-js-club-lighter-small.jpg?nf_resize=fit&amp;w=500   500w,
                  https://infrequently.org/2023/02/the-market-for-lemons/anti-js-js-club-lighter-small.jpg?nf_resize=fit&amp;w=400   400w">
<img src="https://infrequently.org/2023/02/the-market-for-lemons/anti-js-js-club-lighter-small.jpg" alt="Anti JavaScript JavaScript Club" class="preview" decoding="async" loading="lazy">
</source></picture>

<p>We lost a decade to smooth talkers and hollow marketeering; folks who failed the most basic test of intellectual honesty: signposting known unknowns. Instead of engaging honestly with the emerging evidence, they sold lemons and shrunk the market for better solutions. Furiously playing catch-up to stay one step ahead of market rejection, frontend's anguished, belated return to quality has been hindered at every step by those who would stand to lose if their <a href="https://joshcollinsworth.com/blog/self-fulfilling-prophecy-of-react">false premises and hollow promises</a> were to be fully re-evaluated.</p>
<p>Toxic mimicry and recalcitrant ignorance must not be rewarded.</p>
<p>Vendor's random walk through frontend choices may eventually lead them to be right twice a day, but that's not a reason to keep following their lead. No, we need to move our attention back to the folks that have been right all along. The people who never gave up on semantic markup, CSS, and progressive enhancement for <em>most</em> sites. The people who, when slinging JS, have treated it as special occasion food. The tools and communities whose culture puts the user ahead of the developer and hold evidence of doing better for users in the highest regard.</p>
<p>It's not healing, and it won't be enough to nurse the web back to health, but tossing the Vercels and the Facebooks out of polite conversation is, at least, a start.</p>
