title: Keep it simple, stupid le plus longtemps possible
url: https://sklein.xyz/garden/020-keep-it-simple-stupid-le-plus-longtemps-possible/
hash_url: 646ebfa25432bc6b653e524b52d25c9a

<p>Depuis quelques années, j’utilise les mots suivants pour décrire mon mindset de développement :</p>
<blockquote>
<p>« Je suis au maximum le principe KISS (<a href="https://fr.wikipedia.org/wiki/Principe_KISS">Keep it simple, stupid</a>), j’écris le code le plus direct possible
et quand j’ai trop de “douleur” alors je refactor mon code et j’y ajoute le minimum de sophistication indispensable »</p>
</blockquote>

<p>Ici j’utilise le mot “douleur” dans le sens de <a href="https://hn.algolia.com/?dateRange=all&amp;page=0&amp;prefix=false&amp;query=pain%20point&amp;sort=byPopularity&amp;type=story">“a pain point”</a> en anglais : quelque chose de casse-pieds, pénibilité, grosse difficulté…</p>
<h2 id="partie-1-la-quete-du-code-parfait" tabindex="-1">Partie 1 : la quête du code “parfait” </h2>
<p>Pendant la plus grande première partie de ma vie de développeur, j’ai essayé d’écrire le code le plus “propre” possible.</p>
<p>Cela passait par :</p>
<ul>
<li>Découper le code en petites fonctions</li>
<li>Éviter toute duplication de code (mon plus grand combat était le <a href="https://en.wikipedia.org/wiki/Copy-and-paste_programming">copy-and-paste programming</a>, j’ai embêté beaucoup de monde avec cela)</li>
<li>Organiser le plus logiquement possible l’arborescence des dossiers</li>
<li>Créer le maximum de classes (programmation objet) avec le maximum de niveau d’héritage (à l’époque c’était cool de faire de l’héritage)</li>
<li>Utiliser les <a href="https://en.wikipedia.org/wiki/Design_Patterns">Design Patterns</a> (j’ai aussi creusé la <a href="https://fr.wikipedia.org/wiki/Programmation_orient%C3%A9e_aspect">programmation orientée aspect</a>…)</li>
<li><a href="https://en.wikipedia.org/wiki/Don%27t_repeat_yourself">Don’t repeat yourself</a></li>
<li>Découper mon projet en de petits projets ou librairies réutilisables</li>
<li>Multiplier le nombres de repositories (Git…)</li>
<li>Écrire du code pérenne dans le temps</li>
<li>J’essayais d’écrire tout de suite le code dans sa version finale, devoir intervenir à nouveau sur du code était pour moi un échec, la preuve d’un mauvais choix de design.</li>
</ul>
<p>En résumé, un beau code était pour moi un code “intelligent” avec beaucoup de sophistication.</p>
<h2 id="partie-2-la-prise-de-conscience-le-trop-de-tout" tabindex="-1">Partie 2 : la prise de conscience, le “trop de tout” </h2>
<p>Il y a un peu moins de dix ans, j’ai commencé à prendre le chemin inverse, quand j’ai réalisé que je passais mon temps à chercher le code “parfait”.</p>
<ul>
<li>je me suis rendu compte que je passais énormément de temps à découper mon code en petites fonctions, ce qui impliquait :
<ul>
<li>se creuser la tête pour trouver des noms à ces fonctions</li>
<li>de savoir où les placer, dans quels fichiers, dans quels sous dossiers, trouver des noms logiques à ces fichiers et ces dossiers</li>
<li>de réorganiser encore et encore le découpage, le naming au fur et à mesure de l’évolution de l’application</li>
</ul>
</li>
<li>je me suis rendu compte qu’à force d’éviter la duplication de code :
<ul>
<li>que j’avais un code de plus en plus difficile à comprendre</li>
<li>qu’il fallait énormément de temps pour une nouvelle personne pour intervenir sur mon code</li>
</ul>
</li>
<li>j’ai accepté (par expérience) que le code était jetable et que ce n’était pas un échec s’il avait été utile pendant une certaine durée</li>
</ul>
<p>Mon but était-il de faire du code ou de créer un produit rapidement tout en étant de qualité d’un point de vue fonctionnel ?</p>
<h3 id="quest-ce-qui-ma-aide-a-prendre-ce-recul" tabindex="-1">Qu’est-ce qui m’a aidé à prendre ce recul ? </h3>
<h4 id="des-echecs" tabindex="-1">Des échecs </h4>
<p>De 2007 à 2013 j’ai poursuivi la quête du <a href="https://en.wikipedia.org/wiki/Don%27t_repeat_yourself">Don’t repeat yourself</a>, de <a href="https://fr.wikipedia.org/wiki/Pas_de_balle_en_argent">« la balle d’argent »</a> !</p>
<p>Influencé par <a href="https://fr.wikipedia.org/wiki/Ruby_on_Rails">Ruby On Rails</a>, <a href="https://fr.wikipedia.org/wiki/Django_(framework)">Django</a>,
mon rêve pour améliorer ma productivité était de générer automatiquement les applications à partir du modèle de données et des informations de paramétrages des UX.</p>
<p>Ce fut un échec, un <a href="https://github.com/stephane-klein/personnal-notebook/blob/master/003-ne-tonds-pas-de-yaks.md">Yak!</a> perpétuel !</p>
<p>Je passais la majorité de mon temps à :</p>
<ul>
<li>étudier, comprendre les Framework, sélectionner le “meilleur”</li>
<li>embrasser toujours plus de complexité en écrivant des extensions, des plugins pour traiter mes cas particuliers</li>
<li>modifier mes projets pour suivre la monté en version des frameworks</li>
<li>trouver des solutions pour contourner les bugs des frameworks</li>
</ul>
<p>Je me suis trouvé de plus en plus dans des situations où je passais par exemple plus d’une semaine pour modifier un cas particulier d’un simple champ select html,
chose qui m’aurait pris 30min sans framework, sans toutes les couches de magies.</p>
<p>À cela s’ajoute en équipe, les heures et les heures de <a href="https://fr.wikipedia.org/wiki/Troll_(Internet)">trolls</a> de choix de framework.<br>
Chaque développeur a ses préférences, pour tel ou tel framework et tout cela est parfaitement argumentable, car ils ont tous des forces et faiblesses.</p>
<p>C’est suite à cette expérience que j’ai très peur de bâtir une application sur des solutions comme <a href="https://github.com/marmelab/react-admin">React Admin</a>, <a href="https://www.forestadmin.com/">Forest Admin</a> si je sais que je vais devoir les adapter (solutions qui peuvent être très bien pour faire un <a href="https://fr.wikipedia.org/wiki/Preuve_de_concept">POC</a> ou un <a href="https://fr.wikipedia.org/wiki/Produit_minimum_viable">MVP</a>), j’ai très peur de tomber dans un énorme <a href="https://github.com/stephane-klein/personnal-notebook/blob/master/003-ne-tonds-pas-de-yaks.md">Yak!</a>.</p>
<h4 id="des-personnes" tabindex="-1">Des personnes </h4>
<p>Des rencontres, par exemples :</p>
<ul>
<li><a href="https://twitter.com/athoune">Mathieu Lecarme</a> qui m’a dit une fois « j’ai besoin d’intervenir sur …, je constate que c’est de l’horlogerie fine, j’ai besoin d’aide », je me suis dit que j’avais codé quelque chose de bien trop compliqué</li>
<li>Je me souviens de la quête vers le minimaliste dans le code de <a href="https://larlet.fr/david/blog/">David Larlet</a>
<ul>
<li>Est-ce qu’il est possible d’enlever des couches dans la stack ?</li>
</ul>
</li>
<li><a href="https://twitter.com/plafoucriere">Philippe Lafoucrière</a> qui m’a poussé vers du minimalisme, de grosse remise en question en contribuant au code source de Gemnasium
<ul>
<li>Est-ce que mon ORM me fait vraiment gagner du temps ?</li>
<li>Est-ce que je peux me passer de cette librairie ?</li>
</ul>
</li>
</ul>
<p>Ces deux dernières personnes m’ont poussé à réfléchir si je pouvais supprimer des couches, enlever des choses.</p>
<blockquote>
<p>Il semble que la perfection soit atteinte non quand il n’y a plus rien à ajouter, mais quand il n’y a plus rien à retrancher. – <a href="https://fr.wikiquote.org/wiki/Perfection">Antoine de Saint-Exupéry</a></p>
</blockquote>
<h4 id="linfluence-culturelle-de-technos" tabindex="-1">L’influence culturelle de technos </h4>
<p>Depuis 15 ans, je suis très influencé par <a href="https://fr.wikipedia.org/wiki/Zen_de_Python">les 19 principes du Zen de Python</a>, tout particulièrement :</p>
<blockquote>
<ul>
<li>Préfère :
<ul>
<li>l’explicite à l’implicite,</li>
<li>le déroulé à l’imbriqué,</li>
</ul>
</li>
<li>Prends en compte la lisibilité.</li>
<li>Mais, à la pureté, privilégie l’aspect pratique.</li>
<li>Face à l’ambiguïté, à deviner ne te laisse pas aller.</li>
</ul>
</blockquote>
<p>Les choix minimalistes avec peu de sophistication du langage <a href="https://en.wikipedia.org/wiki/Go_(programming_language)">Go</a> a conforté la direction <a href="https://fr.wikipedia.org/wiki/Principe_KISS">KISS</a>.</p>
<h2 id="partie-3-prise-de-decision-le-code-direct-du-code-stupide" tabindex="-1">Partie 3 : prise de décision, le code direct, du code “stupide” </h2>
<p>Depuis quelques années, j’essaie d’écrire un code le plus direct possible.</p>
<p>Mon objectif : diminuer au maximum ma <a href="https://fr.wikipedia.org/wiki/Charge_cognitive">charge cognitive</a>.</p>
<p>J’essaie d’ajouter, une librairie, un service ou une couche d’abstraction uniquement si c’est nécessaire fonctionnellement ou si j’ai beaucoup de “douleur” avec la solution actuelle.</p>
<p>J’essaie de garder un code le plus flat possible (<a href="https://www.thepythoncorner.com/2017/12/the-art-of-avoiding-nested-code/">The art of avoiding nested code</a>, <a href="https://medium.com/@ankushchoubey/clean-code-1-flat-is-better-than-nested-leave-when-not-okay-c09ba74090ef">Flat is better than nested</a>, <a href="http://matthewrocklin.com/blog/work/2019/06/23/avoid-indirection">Avoid Indirection in Code for human readability</a>)</p>
<p>J’essaie de découper mon code uniquement si j’ai trop de “douleur” ou si cela apporte de la valeur fonctionnelle, exemple :</p>
<ul>
<li>je crée une fonction si j’ai besoin de la tester dans un test unitaire, ce qui fini par être le cas si j’ai trop de difficulté sur une section de code</li>
<li>je crée une fonction pour éviter de la duplication en suivant la règle <a href="https://en.wikipedia.org/wiki/Rule_of_three_(computer_programming)">« Rule of three »</a></li>
</ul>

<h2>Ressources</h2>
<ul>
<li><a href="https://en.wikipedia.org/wiki/You_aren%27t_gonna_need_it">YAGNI</a> (You aren’t gonna need it)</li>
<li><a href="https://en.wikipedia.org/wiki/Overengineering">Overengineering</a></li>
<li><a href="https://en.wikipedia.org/wiki/KISS_principle">KISS principle</a></li>
<li><a href="https://en.wikipedia.org/wiki/Don%27t_repeat_yourself">Don’t repeat yourself</a></li>
<li><a href="https://en.wikipedia.org/wiki/Duplicate_code">Duplicate code</a></li>
<li><a href="https://en.wikipedia.org/wiki/Code_reuse">Code reuse</a></li>
<li><a href="https://en.wikipedia.org/wiki/Database_normalization">Database normalization</a></li>
<li><a href="https://en.wikipedia.org/wiki/Rule_of_three_(computer_programming)">Rule of three (computer programming)</a></li>
<li><a href="https://en.wikipedia.org/wiki/Separation_of_concerns">Separation of concerns</a></li>
<li><a href="https://en.wikipedia.org/wiki/Single_source_of_truth">Single source of truth</a></li>
<li><a href="https://josebrowne.com/on-coding-ego-and-attention/">On Coding, Ego and Attention</a></li>
</ul>
