title: Writing with AI
url: https://ia.net/topics/writing-with-ai
hash_url: 291cddda62f18ec9355ec98761b7e9d9
archive_date: 2024-01-21
og_image: https://ia.net/wp-content/uploads/2023/11/keep-it-real-writing-with-AI-1024x475.png
description: When ChatGPT came out one year ago, we wanted to know whether and how it could be used for writing. We put it to the test.
favicon: https://ia.net/wp-content/themes/iA-library/assets/favicons/favicon.ico
language: en_US

<p>When ChatGPT came out one year ago, we wanted to know whether and how it could be used for writing. We put it to the test and came up with a careful answer.</p>
<p>This article is part two of a series about the <a href="https://ia.net/topics/no-feature">history</a>, reason, and the <a href="https://ia.net/topics/ia-writer-7">design</a> of iA Writer 7, our cautious response to AI. In this article, we answer the following five questions:</p>
<ol>
<li><a href="#1">How good is AI for writing?</a></li>
<li><a href="#2">When is AI useful for writing, when is it not?</a></li>
<li><a href="#3">When is it right and when is it wrong to use AI?</a></li>
<li><a href="#4">What is the problem?</a></li>
<li><a href="#5">What should we design?</a></li>
</ol>
<p>You heard it: <em>AI will save the world. AI will destroy the world. AI will become conscious. AI is dumb. AI is genius. AI is starting. AI is over.</em> Some hope that AI will become a robot God, others predict its entropic death. AI is exhausting. If you work in tech, you can’t ignore it. Unfortunately, not every hype is just hype.<sup id="fnref-20899-hype"><a href="#fn-20899-hype" rel="footnote">1</a></sup> So what’s going on? How good is AI for writing, really?</p>
<h2 id="1">1. How good is AI for writing?</h2>
<h3>It’s not <em>that</em> good</h3>
<p>It is plausible that AI will improve, but no one knows what the future brings. Right now, if you have higher expectations than just getting writing done one way or the other, writing with ChatGPT is not <em>that</em> good.</p>
<p>Observing and experimenting with AI, we were not the only ones to find both pleasure and pain, benefits and downsides in AI-driven writing.<sup id="fnref-20899-obvious"><a href="#fn-20899-obvious" rel="footnote">2</a></sup> Some of the dangers like hallucinations and false logic are observable and easily reproducible. Others like loss of voice and common trust in writing are long-term issues, and some plausible traps are somewhat speculative. Currently, it is not that good in terms of:</p>
<ul>
<li><em>Quality:</em> By itself, it makes a lot of mistakes and thus needs careful supervision to stay accurate and meaningful</li>
<li><em>Voice:</em> Letting a machine speak in your place makes you sound and feel like a tool</li>
<li><em>Satisfaction:</em> Having a robot ghost write for you makes you learn, grow, and achieve as much as winning in Chess with a computer</li>
</ul>
<p>And it’s <em>not that good</em> morally either: Why should someone bother to read what you didn’t bother to write?</p>
<h3>It’s not <em>that</em> bad either</h3>
<p>AI can write code, articles, books, emails, and it can do math. Sometimes it’s accurate, sometimes it makes up stuff. It <em>always</em> needs supervision. Left to its own devices, it can make the most ridiculous mistakes and make you look like a complete fool.</p>
<p>It’s both shockingly bad and shockingly good at explaining difficult matters. ChatGPT can explain Kant, the Relativity Theory and it can translate Cuneiform documents at a frightening speed. Mind you: You can <em>not</em> trust it. It has no conscience in any way. It <em>does not understand</em> what it does, and it <em>does not know what is good or bad</em>.</p>
<p>But if you get stuck reading Aristotle’s Metaphysics in ancient Greek, it can clear things up. And even—well, <em>especially</em>—when it’s wrong, it can give you a new perspective. It’s a great learning and dialogue partner. <sup id="fnref-20899-teacher"><a href="#fn-20899-teacher" rel="footnote">3</a></sup> In spite of all of its downsides, AI is cheap, fast, and painless.</p>
<h3>Killer quality convenience</h3>
<p>Despite the incessant hype, fear and hysteria, using AI for writing comes with a killer quality: convenience. It writes faster, clearer, and with fewer typos than most high school students. It’s comparable to early chess computers that quickly beat average players but took some time to reach the grand master level. When you look at how technology evolved over the last 20 years, you know that convenience is <em>the</em> key factor of success.<sup id="fnref-20899-convenience"><a href="#fn-20899-convenience" rel="footnote">4</a></sup></p>
<p>And due to its convenience, we believe that using AI for writing will likely become as common as using dishwashers, spellcheckers, and pocket calculators. The question is: <em>How will it be used?</em> Like spell checkers, dishwashers, chess computers and pocket calculators, writing with AI will be tied to varying rules in different settings.<sup id="fnref-20899-rules"><a href="#fn-20899-rules" rel="footnote">5</a></sup></p>
<p>In order to cope with this <em>new reality for writing</em> we needed to observe and think through the new writing process. We needed to test it up and down, left and right, back and forth, copy and paste, recopy and paste back, through and through.</p>
<h2 id="2">2. When is AI useful for writing, when is it not?</h2>
<h3>Not useful: Think less</h3>
<p>AI turns thoughtful tasks into thoughtless ones. Not thinking is a recipe for disaster. Independent of how AI evolves, letting AI write in our place is a Sci-Fi catastrophe classic.</p>
<p>Thinking less where thinking is key, is not advisable, unless you delegate it to another better-thinking person you trust. And yet. In our initial assessment we were missing a key benefit. The true benefit of writing with AI would only reveal itself through intense usage.</p>
<p>Writing is not about getting letters on a page. It’s not about getting done with text. It’s finding a clear and simple expression for what we feel, mean, and want to express. Writing is thinking with letters. Usually we do this alone. With AI, you write in dialogue. It comes with a chat-interface, after all. So, don’t just write commands, talk to it.</p>
<p><figure class="compare">
<img decoding="async" src="https://ia.net/wp-content/uploads/2023/11/ask-question-AI.png" alt class="macOS littleshadow"></figure></p>
<figcaption><b>Ask me:</b> You can let AI ask you questions about your text. Then copy-paste your answers into an article. A classic writing technique, modernized.</figcaption>
<p>
<figure class="compare">
<img decoding="async" src="https://ia.net/wp-content/uploads/2023/11/ai-is-patient.png" alt class="macOS littleshadow"></figure></p>
<figcaption><b>AI is patient:</b> It will continue asking you until you tell it to stop. No human is that patient with your ideas.</figcaption>
<p></p>
<h3>Useful: Think more</h3>
<p>Writing in dialogue with an artificial companion was unexpected, inspiring and exciting. Using AI in dialog over a writing task was a truly new experience. Using AI as a dialogue partner during preparation, when stuck, and in editing proved surprisingly useful. And that thing is patient like no one else! It never gets tired of you.</p>
<p>Thinking in dialogue is easier and more entertaining than struggling with feelings, letters, grammar and style all by ourselves. Using AI as a writing dialogue partner, ChatGPT can become a catalyst for clarifying what we want to say. Even if it is wrong.<sup id="fnref-20899-scrutinize"><a href="#fn-20899-scrutinize" rel="footnote">6</a></sup> Sometimes we need to hear what’s wrong to understand what’s right.</p>
<p>AI often makes a lot of factual and logical mistakes. Mistakes, if identified, can help you think. Seeing in clear text what is wrong or, at least, what we <em>don’t</em> mean can help us set our minds straight about what we <em>really</em> mean. If you get stuck, you can also simply let it ask you questions. If you don’t know how to improve, you can tell it to be evil in its critique of your writing:</p>
<p><figure class="compare">
<img decoding="async" src="https://ia.net/wp-content/uploads/2023/11/ai-normal-critique.png" alt class="macOS littleshadow"></figure></p>
<figcaption><b>What do you think?</b> Usually ChatGPT tries to be nice. This type of critique may be motivating for beginners, but more experienced writers need more spice.</figcaption>
<p>
<figure class="compare">
<img decoding="async" src="https://ia.net/wp-content/uploads/2023/11/ai-be-evil.png" alt class="macOS littleshadow"></figure></p>
<figcaption><b>“Be as evil as you can”:</b> If you want to learn something you need to make it hurt. If it hurts it has a point, if not, you’re ready to publish.</figcaption>
<p></p>
<p>AI can and will ruin your voice and credibility if you lazily let it write in your place. <em>As writers we can not allow AI to replace our own thinking. We should use it to simulate the thinking of a missing dialogue partner.</em> To write better, we need to think more, not less.</p>
<h2 id="3">3. When is it right and when is it wrong to use AI?</h2>
<p>When it comes to morals, each and everyone sees things a bit differently. But then again, most people agree on the very basics of good and bad. Most people around the globe will agree that stealing, lying and killing are bad habits. Is there anything that corresponds to stealing, lying and killing in writing?</p>
<h3>It’s a spectrum</h3>
<p>Let’s look at some examples. Ask yourself how you judge someone who uses AI on the following spectrum, as:</p>
<ol>
<li>A discussion partner to prepare writing</li>
<li>A spellchecker, grammar checker to fix technical errors</li>
<li>A fact checker, to get rid of overlooked factual errors</li>
<li>A friend that helps out when you can’t find the right word that is on the tip of your tongue</li>
<li>A partial ghostwriter</li>
<li>A full-on ghostwriter, but then you check what was written</li>
<li>A full-on ghostwriter without checking what was written</li>
<li>A full-on ghost reader and ghostwriter without checks and balances</li>
</ol>
<p>It’s not easy to judge the very top and the very end of the list. Likely, different people will have different ethical standards in the middle part. Some will have different standards for themselves and for others and have no problem with that. In other words, at the extremes it’s clear, in the middle it’s up to us. There are limits to what is right and wrong.</p>
<p><figure class="compare">
<img decoding="async" src="https://ia.net/wp-content/uploads/2023/11/ghost-AI-writer.png" alt class="macOS littleshadow"></figure></p>
<figcaption><b>Ghost writer:</b> Why should someone bother to read what you didn’t bother to write?</figcaption>
<p>
<figure class="compare">
<img decoding="async" src="https://ia.net/wp-content/uploads/2023/11/enter-dialog-GPT-writer.png" alt class="macOS littleshadow"></figure></p>
<figcaption><b>Dialog partner:</b> Talk to AI about your writing plans. Discuss style, length, logic, opening and summary. </figcaption>
<p></p>
<h3>Extremes and grey zone</h3>
<p>It’s not that hard to identify which usage is generally sound and which is not. Just compare usage with AI to how we dealt with similar issues before AI.</p>
<ul>
<li>Discussing our writing with others is a general practice and regarded as universally helpful; honest writers honor and credit their discussion partners</li>
<li>We already use spell checkers and grammar tools</li>
<li>It’s common practice to use human editors for substantial or minor copy editing of our public writing</li>
<li>Clearly, using dictionaries and thesauri to find the right expression is not a crime </li>
</ul>
<p>Letting someone else write part of our work is where it gets complicated. And if you look at the way that quoting works in academic writing, quoting a single expression you did not come up with by yourself is, academically speaking, unacceptable. Even transcribing a thought that you owe to another person is not allowed. Outside of academic writing, it’s up to us as writers to decide how much artificiality we are comfortable using and how transparent we want to be about it.</p>
<h2 id="4">4. What is the problem?</h2>
<h3>Entropy: Quoting quoted quotes</h3>
<p>You don’t need to be an academic to see that we shouldn’t pretend to know what we don’t understand, say what we don’t mean, express what we don’t feel. The growing entropy of AI feeding on itself already not only creates issues of social trust and conscience, it also creates technical issues. How about trying to address the entropy instead of increasing it further?</p>
<p>Artificial text is a statistical mashup of human quotes. When we quote AI, we quote quotes. We quote a Bircher muesli of quotes, write over it, and then feed it back into the AI system. There our input gets rehashed again. The way it currently works, AI is more likely to reach lukewarm entropy than ice-cold super-intelligence.</p>
<p>Using AI in the editor replaces thinking. Using AI in dialogue increases thinking. Now, how can connect the editor and the chat window without making a mess? Is there a way to keep human and artificial text apart?</p>
<h3>The key problem: Using AI text is patchwork</h3>
<p>If you do use ChatGPT you will inevitably tend to blend AI-generated text with your own, blurring the lines of authorship. Here is what happens if you use ChatGPT inside a text editor:</p>
<ol>
<li>Write a prompt</li>
<li>Edit the prompt</li>
<li>Try to remember what you wrote yourself</li>
</ol>
<p>Off voice: you won’t remember. Again: Simply avoiding AI integration doesn’t address the problem at all. People will use ChatGPT together with our apps. People will paste robot text, and then edit and forget what is theirs and what is borrowed. <sup id="fnref-20899-mixed"><a href="#fn-20899-mixed" rel="footnote">7</a></sup></p>
<h2 id="5">5. What should we design?</h2>
<h3>Trust and Authorship</h3>
<p>Honesty with others begins with honesty with ourselves, recognizing what is ours and what is borrowed. This starts in the writing process. We need to discern human voice and robot text. Current text editors don’t allow you to easily keep track of what is yours and what isn’t, unless you compare documents or risk high blood pressure with MS Word’s <em>Track Changes</em>.</p>
<p><figure class="compare">
<img decoding="async" src="https://ia.net/wp-content/uploads/2023/11/track-changes.png" alt class="macOS littleshadow"></figure></p>
<figcaption><b>Track Changes:</b> Does the job, but it’s a lot of work, and it comes with so much noise that you quickly forget what you wanted to say.</figcaption>
<p>
<figure class="compare">
<img decoding="async" src="https://ia.net/wp-content/uploads/2023/11/ai-with-kaleidoscope.png" alt class="macOS littleshadow"></figure></p>
<figcaption><b>Compare Versions:</b> Does the job as well, and it’s even more work than “Track Changes.” Not a reasonable way to handle the problem.</figcaption>
<p></p>
<p>Making the difference between humans and machines can’t and shouldn’t be delegated to a machine. It can’t be delegated to anyone other than the writer. It’s up to you, the writer, to be transparent about what is yours and what is not. Only you really know how much you used AI to write. Honesty is up to you. It’s up to us to make it easier for you to discern.</p>
<h3>Keep it simple</h3>
<p>In a time of big claims and exaggerated promises, we have to keep things simple. And that gets very hairy very quickly. We want to offer a straightforward solution for people who want to use AI and at the same time stay in control of what they write.</p>
<ul>
<li>How do we show the difference between artificial and human text without making a mess?</li>
<li>How do we save it in plain text without adding tons of markup? </li>
<li>How do we make human and artificial text discernible as we write without adding a lot of methodical, technical and graphical complexity?</li>
</ul>
<p>At first we thought that we should simply quote AI like we quote other authors. <em>Maybe we could use single straight quotes for AI text, or something like that.</em> But we soon found out that markup doesn’t work at all. Quoting AI is not the same as quoting people. When we use artificial text, we don’t necessarily quote entire sentences or paragraphs. What happens is nitty-gritty patchwork. And ideally, in the end we edit all AI away. How should that work? This is a riddle we had to solve in design and code. Stay tuned to see our solution.</p>
<p><em>This article is part two of a series about the <a href="https://ia.net/topics/no-feature">history</a>, reason, and the <a href="https://ia.net/topics/ia-writer-7">design</a> of iA Writer 7, our cautious response to AI.</em></p>
