title: The full stackoverflow developer
url: https://www.christianheilmann.com/2015/07/17/the-full-stackoverflow-developer/
hash_url: dd3b7b13443bac5d9fe0011426026429

<p>In a few talks and interviews I lamented about a phenomenon in our market that’s always been around, but seems to be rampant by now: the one of <strong>the full stackoverflow developer</strong>. <a href="https://twitter.com/stephenhay/status/621953048986976256">Prompted by Stephen Hay on Twitter</a>, I shall now talk a bit about what this means.</p>

<p><img src="http://christianheilmann.com/wp-content/uploads/2015/07/2322086926_a8a6e8ac7f_o.png" alt="copy and paste"/></p>

<p><strong>Full Stack Overflow developers work almost entirely by copying and pasting code from Stack Overflow instead of understanding what they are doing. Instead of researching a topic, they go there first to ask a question hoping people will just give them the result.</strong></p>

<p>In many cases, this works out. It is amazing what you can achieve by pasting things you don’t understand, that people who know what they are doing put out there.</p>

<p>I am not having a go at Stackoverflow here. It is an incredible resource and it is hard to create a community like this and not drown in spam and mediocrity (trust me, I am admin on several technical Facebook groups).</p>

<p><strong>We had that problem for a long time</strong>. I challenge anyone learning <span class="caps">PHP</span> not simply copying the code examples in the notes. <a href="http://muppets.disney.com/">W3Schools</a> for years gave us the answers we wanted, but didn’t need. Heck, even <a href="http://www.scriptarchive.com/">Matt’s Script Archive</a> is probably the source for many a spam mailer as people used <code>formmail.pl</code> without knowing what it does.</p>

<p>I am, however, worried about how rampant this behaviour is today. Of course, it is understandable:</p>

<ul>
	<li>Creating something is more fun than reading up on how to create something</li>
	<li>Using something that works immediately, even if you don’t know how it does it, feels better than encountering the frustration of not being able to fix something.</li>
	<li>You feel like you cheated the system – shortcuts are fun, and makes you feel like you’re cleverer than all those chumps who spend all this time learning</li>
	<li>Our job is mainstream and there is a massive need for developers. The speed of how we are asked to deliver has massively increased. People want results quicker, rather than cleaner.</li>
</ul>

<p>We, as a community, are partly to blame for breeding this kind of developer:</p>

<ul>
	<li>When we answer questions, we tend to give the solution instead of analysing what the person really needs. This is much more work, so we tend to avoid it.</li>
	<li>Posting the “one true solution” and winning a thread on StackOverflow feels great – even if we have no plan whatsoever to come back to it later if it turns out not to be such a good idea any longer as the environment changed</li>
	<li>Getting recognition, Karma and upvotes for giving the solution is much easier than getting it for being the person who asks the right questions to get to the source of the problem</li>
	<li>It is easy to lose patience with getting the same questions over and over again and a “just use jQuery” is easy to paste</li>
</ul>

<h2>So what? Why is it a problem if people are faster and more effective in releasing products?</h2>

<p>Of course, you can call me a grumpy old so-and-so now and tell me that the concept of learning the basics in software is an outdated concept. The complexity of today’s products makes it almost impossible to know everything and in other, highly successful environments using lots of packages and libraries is par for the course. Fine, although we <a href="http://www.nytimes.com/2015/07/09/business/dealbook/new-york-stock-exchange-suspends-trading.html?_r=0">seem to be understanding</a> that <a href="https://medium.com/message/everything-is-broken-81e5f33a24e1">software as a whole might be more broken</a> than we care to admit, and this might be one of the causes.</p>

<p>There are several problems with full stack overflow development:</p>

<ul>
	<li>It assumes the simplest answer with the most technical detail is the best. This is dangerous and can result in many a copied and pasted example which has a lot of issues surviving for years and years on the web.</li>
	<li>The most copy-able answer being used, upvoted and linked to means that better solutions that fix its issues are much less likely to succeed in replacing them. There is no “digging deeper”, so even important fixes will fall under the radar.</li>
	<li>Any expert community is most likely to have a lot of assumptions as to what makes up a “professional environment”. That means that answers giving in those communities are very likely to work in a great, new and complex developer setup but are not necessarily useful for our end users out there. It is very easy to add yet another library or npm package or bootstrap solution to a project, but it adds to the already full-to-the-brim landfill of outdated code on the web.</li>
	<li>It perpetuates the fondness we have for tersity over writing understandable code. The smallest solution – however unreadable – is always the one that wins the thread. As people copy and paste them without understanding, that’s fine by them. For debugging and maintenance, however, it is the worst solution. A great example for this is <a href="http://www.codereadability.com/javascript-default-parameters-with-or-operator/">the use of || for default parameters</a>. Short isn’t better, it is just less work.</li>
	<li>It cheapens our craft. If we, as the people delivering the work, don’t have any respect for it and simply put things together and hope they work, then we shouldn’t be surprised if our managers and peers don’t see us as professionals.</li>
</ul>

<p>The biggest problem, however, is that it is bad for the developers themselves.</p>

<h2>Finding pride in your work is the biggest reward</h2>

<p>Going on the web, finding a solution and copying and pasting it is easy – too easy. There is no effort in it, and it is not your work – it is someone elses. Instead of being proud of what you achieved, you are more likely to stress out as you don’t want to be found out as a phoney who uses other people’s work and sells it as your own.</p>

<p>Repetition is anathema to a lot of developers. Don’t repeat yourself (DRY) is a good concept in code, but for learning and working it is a terribly idea. We need repetition, to build up muscle memory. The more you repeat a task, the easier it gets and your body does it without you having to think about it.</p>

<p>When you started driving a car, you probably sat down on the seat and got utterly overwhelmed by all the gears, levers, pedals and things to pay attention to. After a while, you don’t even think about what you are doing any longer, and even switching from UK to other cars is not an issue. Failing and learning from it is something we retain much better than simply using something. We put more effort in, it feels more worthy.</p>

<p><a href="https://www.ted.com/talks/dan_ariely_what_makes_us_feel_good_about_our_work?language=en">Dan Ariely’s <span class="caps">TED </span>Talk “What makes us feel good about our work”</a> has some incredibly good points about that topic:</p>



<p>Recognition is what we crave as humans. And we can’t get recognition if we don’t own what we do. You can totally get by copying and pasting and using solution after solution and abstraction upon abstraction. But, sooner or later, you will feel that you are not achieving or creating anything. Many developers who worked like that burned out quickly and stopped developing, stopped caring. And that is a big loss as you might be the person to come up with the next great idea that changes things for a lot of us. That’s an option you shouldn’t rob yourself of. Don’t be a full stackoverflow developer. You deserve to be better.</p>
