title: The Three Kinds of Organizational Power
url: https://jacobian.org/2021/mar/15/organizational-power/
hash_url: 231eb74b73be464d2cd8fc8e3a128e55

<p>Within an organization, there are three kinds of power structures: <strong>role power</strong>, <strong>relationships</strong>, and <strong>expertise</strong>.</p><p>Understanding these kinds of power — how they’re built; how they’re wielded; ethically and otherwise; what they can and can’t accomplish — is key to understanding organizations at a systemic level and maximizing your effectiveness at work.</p><h2 id="what-do-i-mean-by-power-and-why-are-am-i-talking-about-it">What do I mean by “power” and why are am I talking about it?</h2><p>First, I should define what I mean by “power”. I use the term in a sense fairly close to its <a href="https://en.wikipedia.org/wiki/Power_(physics)">definition in physics</a>. In physics, power is “the amount of energy transferred … per unit of time”. More power means more work performed per unit of time; less power: less work. We can think about power in an organizational sense similarly: power, organizationally-speaking, is the ability to get more work done in a given unit of time. More colloquially, power is the ability to get shit done.</p><p>For many, talking about power bluntly can feel gross. We tend to associate “power” with coercion or manipulation, but since the majority of folks are ethical and kind, we don’t love thinking about power. We especially don’t want to think about <em>our</em> power.</p><p>However, there’s a reason why I include <a href="https://www.jofreeman.com/joreen/tyranny.htm">The Tyranny of Structurelessness</a> on <a href="https://jacobian.org/2018/may/2/engmanager-reading-list/">my list of required reading for new managers</a>: power exists within organizations whether we pay attention to it or not. Even in what Jo Freeman calls “structureless” organizations – what we’d now call “flat” organizations – power structures exist whether we want to think about them or our place in them or not.</p><p>But power, at least from an organizational theory standpoint, isn’t about manipulation or getting people to do work they wouldn’t otherwise<sup id="fnref:1"></sup>. Power is an organizational system – the ability for the organization to get work done – that is vested in individuals (because organizations are made up of individuals). In healthy organizations, “power” manifests as a group of people, all aligned and doing their jobs, and all delivering value for the organization together. More on this thought in the section on <a href="#relationship-power">relationship power</a>.</p><p>That’s why I think it’s important for everyone to understand how power exists and manifests inside organizations. Sociopaths already know this stuff, and use it to their advantage (and everyone elses’ detriments). I think it’s critical that empathetic, caring, ethical people understand power, too, so they can get stuff done in an empathetic, caring, ethical way.</p><h2 id="role-power">Role Power</h2><p>When we talk about power at work, this is probably the first thing that comes to mind. Role power is the power vested in you by your role – by your position on the org chart. Fundamentally, it’s the power of getting to use “we”: the power to speak on behalf of the organization. An individual contributor can’t say “we’re all getting new laptops” (well, they can, but it probably won’t happen); the CEO certainly can.</p><p>Role power also manifests in the power to tell subordinates what to do. When your boss asks you to do something, there’s always an implicit “do this because I’m your boss” in that request. All else being equal, you’re more likely to do the thing if your boss asks than if someone else does.</p><p>However, role power is limited – more limited than most people think. I would say that role power only accounts for maybe 20% of a person’s overall power organizationally<sup id="fnref:2"></sup>. Yes, a boss can, well, boss people around&amp;mldr; but the effectiveness of this is quite limited. Orders lead to compliance, but not alignment; a manager who orders people around will get only the minimum out of their staff. They’ll be causing burnout and resentment, which will sooner or later cause that manager to fail.</p><p>Further, there’s not that much that you can accomplish through role power alone. When I first became a manager, I had grand expectations about how much more I’d be able to get done. Our team had been struggling with a difficult technical transition: I thought our organization wasn’t making it a high enough priority, and I thought that people on the team were approaching the work poorly. I thought, “now that I’m the boss they’ll have to listen to me!”</p><p>I was so wrong.</p><p>I started ordering people around and arguing with my new peers on the management team. I didn’t quite get to the point of shouting “respect my authority!” – but I did get embarrassingly close to that. This didn’t just fail: it made the situation worse. My direct reports, who had respected me as a peer, lost that respect when I became their boss. My new peers, my fellow managers, stopped paying attention to me. I left the organization about a year later, tail between my legs.</p><p>Role power is, somewhat contradictorily, more powerful the less you use it. If you get work done through other forms of power 99% of the time, when you <em>do</em> finally say “do this because I’m your boss”, it lands with force – and the relationship is strong enough to absorb that forcefulness. On the other hand, using role power too often damages relationships – and as we’ll see, relationship power is the most effective form of organizational power.</p><p>Role power can only be built in one way: by getting promoted, moving up the org chart. This also diminishes its value: you can’t build role power unilaterally; you only have as much or as little as the org vests in you. It’s a fixed thing; the others are not.</p><h2 id="relationship-power">Relationship Power</h2><p>While role power is overemphasized, <strong>relationship power</strong> is often underestimated – but it’s by far the most effective sort of organizational power. If role power accounts for maybe 20% of someone’s total power, relationship power is something like 70%-80%.</p><p>Relationship power is simply the ability to get work done through your relationships with others. It’s all the work that gets done because people and teams know and understand each other, work together effectively, and are happy to help each other out because they want others to be successful, too.</p><p>If there’s one thing you take away from this article I hope it’s this: <strong>spend more time building relationships at work</strong>. If you build strong relationships with your colleagues, your work will be smoother, happier, and more effective.</p><p>An example of how relationship power works:</p><p>I’ve had several roles where I’ve been responsible for developing budgets. Each time, I’m usually the first among my peers to have my budget approved; it takes me mere days while my peers spend weeks or months going back and forth on their budgets. Is this because I’m just really good at budgets? Well, maybe – but more likely, it’s because I’ve laid the groundwork for a smooth approval far in advance. I’ve met everyone involved in the budget approval chain – a finance person or three, maybe our Head of Operations or COO. I get coffee with them, get to know them and their jobs, really understand what they’re looking for. I make sure they know that I’m happy to help them out in their jobs. Often, I can: if I develop a bit of light automation, or some Excel/Google Sheets macros, I can save finance folks boatloads of time.</p><p>Importantly: I don’t do this because I’ll later need something from them! This isn’t a <a href="https://www.shmoop.com/quotes/someday-and-that-day-may-never-come.html">Don Corleone moment</a> – that would be ugly and manipulative. This is something I do with <em>all</em> the people I need to work with because I genuinely want to know and understand them and their jobs<sup id="fnref:3"></sup>. I offer to help because helping other people be more effective in their jobs is good for the whole organization (and it feels good to help, individually). Having a good relationship means working together is easy and effective, and I want that with everyone.</p><p>So, when budget season comes, I find it pretty easy. My first draft will be far closer to done because I understand what the other stakeholders want and can anticipate some of their questions. Because we’ve already worked together, if there’s a problem they know it’ll be easy to pick up the phone and tell me about it.</p><p>Importantly, relationship power functions <em>within</em> reporting chains too! Sure, you certainly are <em>allowed</em> to order a direct report to do something, but as I’ve noted above, this is ineffective. If instead, you understand your reports well – what they like, what they don’t, what motivates them – and they understand you and the team – what you need, what success looks like for the team, how the team’s work fits into the greater org – you can ask them for work within that context and they’ll be happy to do it. You might not even need to ask; they might just know!</p><p>This is why empathy and emotional intelligence are such critical skills for managers: understanding other people, being able to reason about how they think and what motivates them, and genuinely caring about their well-being all make it easier to work with them to get things done.</p><p>You build relationship power by&amp;mldr; building relationships. This begins with talking to them. There’s a reason why one-on-ones are <em>the</em> cornerstone of good management practice: 1:1s mean scheduled time for building relationships. Regular 1:1s lead to better relationships and this builds relationship power for both the manager and the report!</p><p>You also build relationship power outside your reporting chain through talking to folks – regular meetings, coffee breaks, chats over lunch, and so forth. In <a href="https://www.amazon.com/dp/B015VACHOK">High Output Management</a> (one of the all-time best management books), Andy Grove writes about how much time he spends simply walking around and talking to people. It can seem surprising that an executive would spend so much time just listening, but</p><blockquote><p>It’s obvious that your decision-making depends finally on how well you comprehend the facts and issues facing your business. This is why information-gathering is so important in a manager’s life. Other activities—conveying information, making decisions, and being a role model for your subordinates—are all governed by the base of information that you, the manager, have about the tasks, the issues, the needs, and the problems facing your organization. <strong>In short, information-gathering is the basis of all other managerial work, which is why I choose to spend so much of my day doing it.</strong></p></blockquote><p>(Emphasis mine.)</p><p>You also build relationship power every time you work with someone (assuming you do a good job, of course). This doesn’t necessarily mean doing favors – relationship power isn’t a Machiavellian quid pro quo – it’s simply that when you accomplish something with other people, that relationship gets stronger. One weird thing I’ve found is that sometimes <em>asking for help</em> can build a relationship faster than offering it. Most people want to help their colleagues, and being open about where you’re struggling is a great way to show that it’s safe for them to open up to you.</p><h2 id="expertise-power">Expertise Power</h2><p>Finally, expertise power. Expertise power is the power you have at an organization by being a clear expert in some technology, system, or process. It’s the power that a person (or team) has because they know or understand something better than everyone.</p><p>I have less to say about expertise power than the previous two, mostly because I’ve rarely worked for organizations or in roles where it’s been all that useful<sup id="fnref:4"></sup>. Expertise power is something typically that Staff- or Principal-level engineers have. These positions don’t have the same role power as managers. Both managers and very senior <abbr title="individual contributors">ICs</abbr> fundamentally use relationship power to get stuff done, but senior <abbr title="individual contributors">ICs</abbr> augment that with expertise power rather than role power.</p><p>If you’re interested in learning more about how expertise functions within these sorts of senior <abbr title="individual contributor">ICs</abbr> roles, I suggest Will Larson’s book <a href="https://staffeng.com/book">Staff Engineer: Leadership beyond the management track</a>.</p><h2 id="conclusion">Conclusion</h2><p>Learning to think about organizational power was transformative for my career. Suddenly, I was able to understand why some projects got done – and others withered on the vine. Understanding <em>why</em> relationships are so important helped me make sure I prioritized them appropriately. I doubt I’d be the leader I am today without gaining this understanding. I hope it likewise helps you in your career!</p>
